import wx
from prompt_db import PromptDatabase


class PromptDialog(wx.Dialog):
	def __init__(self, parent):
		super(PromptDialog, self).__init__(parent, title="Manage Prompts")
		self.promptDB = PromptDatabase()
		self.initUI()
		self.loadPrompts()

	def initUI(self):
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		self.promptList = wx.ListBox(self)
		self.promptInput = wx.TextCtrl(self, style=wx.TE_MULTILINE)
		self.addButton = wx.Button(self, label="Add to Database")
		self.removeButton = wx.Button(self, label="Remove from Database")
		self.useButton = wx.Button(self, label="Use Now")
		self.addButton.Bind(wx.EVT_BUTTON, self.onAddPrompt)
		self.removeButton.Bind(wx.EVT_BUTTON, self.onRemovePrompt)
		self.useButton.Bind(wx.EVT_BUTTON, self.onUsePrompt)
		mainSizer.Add(self.promptList, 1, wx.EXPAND | wx.ALL, 5)
		mainSizer.Add(self.promptInput, 1, wx.EXPAND | wx.ALL, 5)
		mainSizer.Add(self.addButton, 0, wx.ALL, 5)
		mainSizer.Add(self.removeButton, 0, wx.ALL, 5)
		mainSizer.Add(self.useButton, 0, wx.ALL, 5)
		self.SetSizer(mainSizer)

	def loadPrompts(self):
		self.promptList.Set(self.promptDB.get_prompts())

	def onAddPrompt(self, event):
		prompt = self.promptInput.GetValue()
		self.promptDB.add_prompt(prompt)
		self.loadPrompts()

	def onRemovePrompt(self, event):
		selection = self.promptList.GetSelection()
		if selection != wx.NOT_FOUND:
			prompt = self.promptList.GetString(selection)
			self.promptDB.remove_prompt(prompt)
			self.loadPrompts()

	def onUsePrompt(self, event):
		# This should trigger the global command to use the prompt immediately
		# For now, we just print the selected prompt
		selection = self.promptList.GetSelection()
		if selection != wx.NOT_FOUND:
			prompt = self.promptList.GetString(selection)
			print(f"Using prompt: {prompt}")
